==> 0.0.1.tar.gz: lib/widgets/spark_form_widget.dart <==
part of spark_widget;

abstract class SparkFormWidget extends StatefulWidget {
  const SparkFormWidget({Key? key, required this.manage}) : super(key: key);

  final FormItemManage manage;
}

abstract class SparkFormWidgetBase<T extends SparkFormWidget> extends State<T> {
  /// form info data
  // SparkFormItemData? formData;

  /// old value record
  dynamic oldValue;

  void handleFormDataChange(dynamic newValue, dynamic oldValue) {
    FormDataChangeNotification(
      newValue: newValue,
      oldValue: oldValue,
    ).dispatch(context);
    // oldValue = widget.manage.controller.text;
  }
}

class FocusPointerScope extends StatefulWidget {
  const FocusPointerScope({
    Key? key,
    this.focusNode,
    required this.child,
    this.shadow = false,
    this.enabled,
    this.decoration,
    this.padding,
    this.margin,
    this.alignment,
    this.constraints,
    this.onTap,
    this.onFocus,
    this.onUnFocus,
  }) : super(key: key);

  final FocusNode? focusNode;

  final Widget child;

  /// only request focus,but not set focus style
  final bool? shadow;

  /// focus enable
  final bool? enabled;

  /// widget decoration
  final FocusDecoration? decoration;

  /// content padding
  final EdgeInsets? padding;

  /// content margin
  final EdgeInsets? margin;

  final AlignmentGeometry? alignment;

  final BoxConstraints? constraints;

  /// on widget tap
  final VoidCallback? onTap;

  /// on focus
  final VoidCallback? onFocus;

  /// on lose focus
  final VoidCallback? onUnFocus;

  @override
  State<StatefulWidget> createState() => _FocusPointerScopeBase();
}

class _FocusPointerScopeBase extends State<FocusPointerScope> {
  FocusNode? _focusNode;

  FocusAttachment? _focusAttachment;

  bool get _isEnabled => widget.enabled ?? true;

  FocusNode get _effectiveFocusNode =>
      widget.focusNode ?? (_focusNode ??= FocusNode());

  /// update focus widget
  _handleFocusChanged() {
    if (_effectiveFocusNode.hasFocus) {
      if (widget.onFocus != null) {
        widget.onFocus!();
      }
    } else {
      if (widget.onUnFocus != null) {
        widget.onUnFocus!();
      }
    }
    setState(() {});
  }

  /// widget can get focus
  bool get _canRequestFocus {
    final NavigationMode mode = MediaQuery.maybeOf(context)?.navigationMode ??
        NavigationMode.traditional;
    switch (mode) {
      case NavigationMode.traditional:
        return _isEnabled;
      case NavigationMode.directional:
        return true;
    }
  }

  _handleTap() {
    if (!_isEnabled) {
      return;
    }
    if (widget.onTap != null) {
      widget.onTap!();
    }
    FocusScope.of(context).requestFocus(_effectiveFocusNode);
  }

  @override
  void initState() {
    _effectiveFocusNode.addListener(_handleFocusChanged);
    // attach focusNode target widget
    _focusAttachment = _effectiveFocusNode.attach(context);
    super.initState();
  }

  @override
  void didUpdateWidget(covariant FocusPointerScope oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.focusNode != oldWidget.focusNode) {
      (oldWidget.focusNode ?? _focusNode)?.removeListener(_handleFocusChanged);
      (widget.focusNode ?? _focusNode)?.addListener(_handleFocusChanged);
      // replace focusNode, rebind context
      _focusAttachment?.detach();
      _focusAttachment = _effectiveFocusNode.attach(context);
    }
    _effectiveFocusNode.canRequestFocus = _canRequestFocus;
  }

  @override
  void dispose() {
    _effectiveFocusNode.removeListener(_handleFocusChanged);
    _focusNode?.dispose();
    _focusAttachment?.detach();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    bool _hasFocus = _effectiveFocusNode.hasFocus && widget.shadow != true;
    FocusDecoration _decoration = widget.decoration ?? const FocusDecoration();
    // use Listener solving competition conflict
    return Listener(
      behavior: HitTestBehavior.opaque,
      onPointerUp: (_e) {
        _handleTap();
      },
      child: Container(
        decoration: BoxDecoration(
          color: _decoration.color,
          image: _decoration.image,
          border:
              _hasFocus ? _decoration.focusBorder : _decoration.unFocusBorder,
          borderRadius: _decoration.borderRadius,
          boxShadow: _decoration.boxShadow,
          gradient: _decoration.gradient,
          backgroundBlendMode: _decoration.backgroundBlendMode,
          shape: _decoration.shape,
        ),
        constraints: widget.constraints,
        padding: widget.padding,
        margin: widget.margin,
        alignment: widget.alignment,
        child: widget.child,
      ),
    );
  }
}

class FocusDecoration {
  const FocusDecoration({
    this.color,
    this.image,
    this.focusBorder,
    this.borderRadius,
    this.unFocusBorder,
    this.boxShadow,
    this.gradient,
    this.backgroundBlendMode,
    this.shape = BoxShape.rectangle,
  });

  final Color? color;
  final DecorationImage? image;
  final BoxBorder? focusBorder;
  final BoxBorder? unFocusBorder;
  final BorderRadiusGeometry? borderRadius;
  final List<BoxShadow>? boxShadow;
  final Gradient? gradient;
  final BlendMode? backgroundBlendMode;
  final BoxShape shape;
}

==> 0.0.2.tar.gz: lib/widgets/spark_form_widget.dart <==
part of spark_widget;

abstract class SparkFormWidget extends StatefulWidget {
  const SparkFormWidget({Key? key, required this.manage}) : super(key: key);

  final FormItemManage manage;
}

abstract class SparkFormStateLessWidget<D> extends StatelessWidget {
  const SparkFormStateLessWidget({Key? key, required this.manage}) : super(key: key);

  final FormItemManage manage;

  D get fieldData => manage.notifier.data;

  void handleFormDataChange(BuildContext context, dynamic newValue) {
    FormDataChangeNotification(
      newValue: newValue,
    ).dispatch(context);
  }
}

abstract class SparkFormWidgetBase<T extends SparkFormWidget, D> extends State<T> {

  D get fieldData => widget.manage.notifier.data;

  FormItemManage get manage => widget.manage;

  void handleFormDataChange(dynamic newValue) {
    FormDataChangeNotification(
      newValue: newValue,
    ).dispatch(context);
  }

  @override
  void initState() {
    widget.manage.notifier.addListener(valueChangeNotify);
    super.initState();
  }

  void valueChangeNotify();

  @override
  void dispose(){
    widget.manage.notifier.removeListener(valueChangeNotify);
    super.dispose();
  }

}

class FocusPointerScope extends StatefulWidget {
  const FocusPointerScope({
    Key? key,
    this.focusNode,
    required this.child,
    this.shadow = false,
    this.enabled,
    this.decoration,
    this.padding,
    this.margin,
    this.alignment,
    this.constraints,
    this.onTap,
    this.onFocus,
    this.onUnFocus,
    this.width,
    this.height,
  }) : super(key: key);

  final FocusNode? focusNode;

  final Widget child;

  /// only request focus,but not set focus style
  final bool? shadow;

  /// focus enable
  final bool? enabled;

  /// widget decoration
  final FocusDecoration? decoration;

  /// content padding
  final EdgeInsets? padding;

  /// content margin
  final EdgeInsets? margin;

  final AlignmentGeometry? alignment;

  final BoxConstraints? constraints;

  /// on widget tap
  final VoidCallback? onTap;

  /// on focus
  final VoidCallback? onFocus;

  /// on lose focus
  final VoidCallback? onUnFocus;

  final double? width;

  final double? height;

  @override
  State<StatefulWidget> createState() => _FocusPointerScopeBase();
}

class _FocusPointerScopeBase extends State<FocusPointerScope> {
  FocusNode? _focusNode;

  FocusAttachment? _focusAttachment;

  bool get _isEnabled => widget.enabled ?? true;

  FocusNode get _effectiveFocusNode =>
      widget.focusNode ?? (_focusNode ??= FocusNode());

  /// update focus widget
  _handleFocusChanged() {
    if (_effectiveFocusNode.hasFocus) {
      if (widget.onFocus != null) {
        widget.onFocus!();
      }
    } else {
      if (widget.onUnFocus != null) {
        widget.onUnFocus!();
      }
    }
    setState(() {});
  }

  /// widget can get focus
  bool get _canRequestFocus {
    final NavigationMode mode = MediaQuery.maybeOf(context)?.navigationMode ??
        NavigationMode.traditional;
    switch (mode) {
      case NavigationMode.traditional:
        return _isEnabled;
      case NavigationMode.directional:
        return true;
    }
  }

  _handleTap() {
    if (!_isEnabled) {
      return;
    }
    if (widget.onTap != null) {
      widget.onTap!();
    }
    FocusScope.of(context).requestFocus(_effectiveFocusNode);
  }

  @override
  void initState() {
    _effectiveFocusNode.addListener(_handleFocusChanged);
    // attach focusNode target widget
    _focusAttachment = _effectiveFocusNode.attach(context);
    super.initState();
  }

  @override
  void didUpdateWidget(covariant FocusPointerScope oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.focusNode != oldWidget.focusNode) {
      (oldWidget.focusNode ?? _focusNode)?.removeListener(_handleFocusChanged);
      (widget.focusNode ?? _focusNode)?.addListener(_handleFocusChanged);
      // replace focusNode, rebind context
      _focusAttachment?.detach();
      _focusAttachment = _effectiveFocusNode.attach(context);
    }
    _effectiveFocusNode.canRequestFocus = _canRequestFocus;
  }

  @override
  void dispose() {
    _effectiveFocusNode.removeListener(_handleFocusChanged);
    _focusNode?.dispose();
    _focusAttachment?.detach();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    bool _hasFocus = _effectiveFocusNode.hasFocus && widget.shadow != true;
    FocusDecoration _decoration = widget.decoration ?? const FocusDecoration();
    // use Listener solving competition conflict
    return Listener(
      behavior: HitTestBehavior.opaque,
      onPointerUp: (_e) {
        _handleTap();
      },
      child: Container(
        decoration: BoxDecoration(
          color: _decoration.color,
          image: _decoration.image,
          border:
              _hasFocus ? _decoration.focusBorder : _decoration.unFocusBorder,
          borderRadius: _decoration.borderRadius,
          boxShadow: _decoration.boxShadow,
          gradient: _decoration.gradient,
          backgroundBlendMode: _decoration.backgroundBlendMode,
          shape: _decoration.shape,
        ),
        width: widget.width,
        height: widget.height,
        constraints: widget.constraints,
        padding: widget.padding,
        margin: widget.margin,
        alignment: widget.alignment,
        child: widget.child,
      ),
    );
  }
}

class FocusDecoration {
  const FocusDecoration({
    this.color,
    this.image,
    this.focusBorder,
    this.borderRadius,
    this.unFocusBorder,
    this.boxShadow,
    this.gradient,
    this.backgroundBlendMode,
    this.shape = BoxShape.rectangle,
  });

  final Color? color;
  final DecorationImage? image;
  final BoxBorder? focusBorder;
  final BoxBorder? unFocusBorder;
  final BorderRadiusGeometry? borderRadius;
  final List<BoxShadow>? boxShadow;
  final Gradient? gradient;
  final BlendMode? backgroundBlendMode;
  final BoxShape shape;

  FocusDecoration copyWith({
    final Color? color,
    DecorationImage? image,
    BoxBorder? focusBorder,
    BoxBorder? unFocusBorder,
    BorderRadiusGeometry? borderRadius,
    List<BoxShadow>? boxShadow,
    Gradient? gradient,
    BlendMode? backgroundBlendMode,
    BoxShape? shape,
  }) {
    return FocusDecoration(
      color: color ?? this.color,
      image: image ?? this.image,
      focusBorder: focusBorder ?? this.focusBorder,
      unFocusBorder: unFocusBorder ?? this.unFocusBorder,
      boxShadow: boxShadow ?? this.boxShadow,
      gradient: gradient ?? this.gradient,
      backgroundBlendMode: backgroundBlendMode ?? this.backgroundBlendMode,
      shape: shape ?? this.shape,
    );
  }
}
